use std::sync::Mutex;
use tokio::sync::Notify;

#[derive(Debug)]
pub(crate) struct LongPoller<T> {
    inner: Mutex<Option<T>>,
    notify: Notify,
}

impl<T> LongPoller<T> {
    pub(crate) fn new() -> Self {
        Self {
            inner: Mutex::new(None),
            notify: Notify::new(),
        }
    }

    pub(crate) async fn wait(&self) -> T {
        loop {
            if let Some(value) = self.inner.lock().unwrap().take() {
                return value;
            }
            self.notify.notified().await;
        }
    }

    pub(crate) fn update(&self, value: T) {
        *self.inner.lock().unwrap() = Some(value);
        self.notify.notify_one();
    }
}
